 
#include "alua_gsmApp.h"       // Header file to this source file 
#include "alua_gsmAT.h"
#include "alua_typedef.h"
#include "alua_config_app.h"
#include <string.h>
#include <stdlib.h>
#include "alua_global.h"
#include "alua_uart.h"
#include "alua_miscfunc.h"
#include "alua_config_hw.h"
#include "alua_gpsApp.h"
#include "alua_gsmMqtt.h"

#define CERT_CHUNK_LEN      64
#define MQTT_BUF_LEN        100


uint8_t fg_mqttInitFlag = 0;
uint16_t fg_16bTemp;
void(*fn_cbMqtt)(void) = NULL;

extern en_gsmState fg_gsmState;
extern st_gpsRMC fg_gpsRmcData;
extern uint8_t fg_gsmNoResetIndex;      //no reset Rx Index
extern uint16_t fg_gsmDelay, fg_gsmRetry;
extern uint8_t fg_gsmFrameReceived, fg_gsmRxCount, fg_gsmRssi;
extern uint8_t fg_gsmFlagInit, fg_gsmCurrStage, fg_gsmLastStage;
extern uint16_t fg_gsmTxCount;
extern uint8_t fg_gsmRxBuf[UART_GSM_BUFFER_LEN], fg_gsmTxBuf[UART_GSM_BUFFER_LEN];
//==============================================================================
void alua_gsmMqttAnalyze(void)
{
    uint8_t l_retVal = 0x01;
    
    fg_gsmState = STATE_GSM_IDLE;
    fg_gsmTxCount = 0x00;
    fg_gsmRxCount = alua_uart2GetRxCount();            //get number of bytes received
    if(!fg_gsmNoResetIndex) alua_uart2ResetIndex();     //clear the uart character counter
       
    
    if(fg_gsmCurrStage != 23)   //publish case 
    {
        //chcek trailing CRLF
        if(!((fg_gsmRxBuf[fg_gsmRxCount - 2] == 0x0D) && (fg_gsmRxBuf[fg_gsmRxCount - 1] == 0x0A) ))
        {
            return;     //not valid frame
        }
    }
    
    switch(fg_gsmCurrStage)
    {
        //-----------------------------------------------
        case 0: 
            if(strstr((const char*) fg_gsmRxBuf, "+CSQ:")) 
            {
                fg_gsmRssi = alua_gsmGetRssi(fg_gsmRxBuf);
                fg_gsmCurrStage++;
            }
            break;
        //-----------------------------------------------
        case 1:
            if (strstr((const char*) fg_gsmRxBuf, "OK")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 2:
            if(strstr((const char*) fg_gsmRxBuf, "OK")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 3:
            if(strstr((const char*) fg_gsmRxBuf, "DEACT")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 4:
             if(strstr((const char*) fg_gsmRxBuf, "OK")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 5:
            if(strstr((const char*) fg_gsmRxBuf, "CONNECT")) 
                fg_gsmCurrStage++;      //go to next step
            else if(strstr((const char*) fg_gsmRxBuf, "exist")) 
                fg_gsmCurrStage += 2;   //go to next certificate
            break;
        //-----------------------------------------------
        case 6:
            if(strstr((const char*) fg_gsmRxBuf, "+QSECWRITE")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 7:
            if(strstr((const char*) fg_gsmRxBuf, "CONNECT")) 
                fg_gsmCurrStage++;      //go to next step
            else if(strstr((const char*) fg_gsmRxBuf, "exist")) 
                fg_gsmCurrStage += 2;   //go to next certificate
            break;
        //-----------------------------------------------
        case 8:
            if(strstr((const char*) fg_gsmRxBuf, "+QSECWRITE")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 9:
            if(strstr((const char*) fg_gsmRxBuf, "CONNECT")) 
                fg_gsmCurrStage++;      //go to next step
            else if(strstr((const char*) fg_gsmRxBuf, "exist")) 
                fg_gsmCurrStage += 2;   //go to next certificate
            break;
        //-----------------------------------------------
        case 10:
            if(strstr((const char*) fg_gsmRxBuf, "+QSECWRITE")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 11:
            if(strstr((const char*) fg_gsmRxBuf, "OK")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 12:
            if(strstr((const char*) fg_gsmRxBuf, "OK")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 13:
            if(strstr((const char*) fg_gsmRxBuf, "OK")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 14:
            if(strstr((const char*) fg_gsmRxBuf, "OK")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 15:
            if(strstr((const char*) fg_gsmRxBuf, "OK")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 16:
            if(strstr((const char*) fg_gsmRxBuf, "OK")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 17:
            if(strstr((const char*) fg_gsmRxBuf, "OK")) fg_gsmCurrStage++;
            break;
        //-----------------------------------------------
        case 18: //MQTT Start
            if(strstr((const char*) fg_gsmRxBuf, "OK"))
            {
                fg_gsmCurrStage++;
                alua_gsmSetTimeout(0x01, 750);  //Max 75 seconds waiting time
                l_retVal = 0x00;    //do not send next at command, but wait for server response
            }
            break;
        //-----------------------------------------------
        case 19:    //wait for MQTOPEN Server Response
            if(strstr((const char*) fg_gsmRxBuf, "+QMTOPEN:"))
            {
                if(strstr((const char*) fg_gsmRxBuf, "0,0"))
                {
                    fg_gsmCurrStage++;  //go to next command
                }
                else
                {
                    fg_gsmCurrStage--;  //return to previous state
                }
                l_retVal = 0x01;    
            }
            break;
        //-----------------------------------------------
        case 20:
            if(strstr((const char*) fg_gsmRxBuf, "+QMTCONN:"))
            {
                fg_gsmCurrStage++; 
            }
            else
            {
                 l_retVal = 0;
            }
            break;
        //-----------------------------------------------
        case 21:    //subscribe
            if(strstr((const char*) fg_gsmRxBuf, "+QMTSUB:"))
            {
                fg_gsmCurrStage++; 
            }
            else
            {
                 l_retVal = 0;
            }
            break;
        //-----------------------------------------------
        //case 22:      //not required 
        //    break;
        //-----------------------------------------------
        case 23:
            if(fg_gsmRxBuf[2] == '>')
            {
                fg_gsmCurrStage++; 
                fg_gsmState = STATE_GSM_INIT;   //to start fast
                l_retVal = 0x00;
            }
            break;
        //-----------------------------------------------
        case 24:
            if(strstr((const char*) fg_gsmRxBuf, "+QMTPUB:"))
            {
                fg_gsmCurrStage = 22;
            }
            else
            {
                l_retVal = 0;
            }
            break;
        //-----------------------------------------------
        default:
            l_retVal = 0x00;
            break;
    }
    
    if(fg_gsmLastStage != fg_gsmCurrStage) 
    {
        fg_gsmLastStage = fg_gsmCurrStage;
        fg_gsmRetry = 0x00;
    }
    
    if(l_retVal)
    {
        fg_gsmState = STATE_GSM_INIT;
        alua_gsmResetTimeout();
    }
}

//============================================================================
void alua_gsmMqttQuery(void)
{
    fg_gsmState = STATE_GSM_IDLE;
    fg_gsmTxCount = 0x00;
    fg_gsmDelay = 30;        //@100ms
    memset(fg_gsmTxBuf, 0, sizeof(fg_gsmTxBuf));
    fg_gsmNoResetIndex = 0x00;
    
    switch(fg_gsmCurrStage)
    {
        //-----------------------------------------------
        case 0:    
            fg_gsmTxCount = sizeof(AT_READ_RSSI);
            memcpy(fg_gsmTxBuf, AT_READ_RSSI, fg_gsmTxCount);
            break;
        //-----------------------------------------------
        case 1:    
            fg_gsmTxCount = sizeof(AT_START_CONTEXT);
            memcpy(fg_gsmTxBuf, AT_START_CONTEXT, fg_gsmTxCount);
            break;
         //-----------------------------------------------
        case 2:
            fg_gsmTxCount = sizeof(AT_SET_APN);
            memcpy(fg_gsmTxBuf, AT_SET_APN, fg_gsmTxCount);
            break;
         //-----------------------------------------------
        case 3:
            fg_gsmTxCount = sizeof(AT_GPRS_DEACT);
            memcpy(fg_gsmTxBuf, AT_GPRS_DEACT, fg_gsmTxCount);
            break;
        //-----------------------------------------------
        case 4:    
            fg_gsmTxCount = sizeof(AT_QMTCFG_SSL);
            memcpy(fg_gsmTxBuf, AT_QMTCFG_SSL, fg_gsmTxCount);
            break;
        //-----------------------------------------------
        case 5: //ca1 header
            fg_gsmTxCount = sizeof(AT_QSECWRITE_CACERT) - 1;
            memcpy(fg_gsmTxBuf, AT_QSECWRITE_CACERT, fg_gsmTxCount);
            alua_miscConv16bToDecAscii(&fg_gsmTxBuf[fg_gsmTxCount], MQTT_AWS_CA1_CERT_LEN);
            fg_gsmTxCount += 4;
            memcpy(&fg_gsmTxBuf[fg_gsmTxCount], ",100", 4);
            fg_gsmTxCount += 5;
            break;
        //-----------------------------------------------
        case 6: //ca certificate
            fg_gsmTxCount = alua_gsmMqttCopyCert(AWS_CERT_CA1);
            fg_gsmTxCount += 1; //one extra required
            break;
        //-----------------------------------------------
        case 7: //client header
            fg_gsmTxCount = sizeof(AT_QSECWRITE_CLIENT) - 1;
            memcpy(fg_gsmTxBuf, AT_QSECWRITE_CLIENT, fg_gsmTxCount);
            alua_miscConv16bToDecAscii(&fg_gsmTxBuf[fg_gsmTxCount], MQTT_AWS_CLIENT_CERT_LEN);
            fg_gsmTxCount += 4;
            memcpy(&fg_gsmTxBuf[fg_gsmTxCount], ",100", 4);
            fg_gsmTxCount += 5;
            break;
        //-----------------------------------------------
        case 8: //client certificate
            fg_gsmTxCount = alua_gsmMqttCopyCert(AWS_CERT_CLIENT);
            fg_gsmTxCount += 1; //one extra required
            break;
         //-----------------------------------------------
        case 9: //private header
            fg_gsmTxCount = sizeof(AT_QSECWRITE_USER) - 1;
            memcpy(fg_gsmTxBuf, AT_QSECWRITE_USER, fg_gsmTxCount);
            alua_miscConv16bToDecAscii(&fg_gsmTxBuf[fg_gsmTxCount], MQTT_AWS_PRIVATE_CERT_LEN);
            fg_gsmTxCount += 4;
            memcpy(&fg_gsmTxBuf[fg_gsmTxCount], ",100", 4);
            fg_gsmTxCount += 5;
            break;
        //-----------------------------------------------
        case 10: //user certificate
            fg_gsmTxCount = alua_gsmMqttCopyCert(AWS_CERT_PRIVATE);
            fg_gsmTxCount += 1; //one extra required
            break;
        //-----------------------------------------------
        case 11: //ca config 
            fg_gsmTxCount = sizeof(AT_QSSLCFG_CACERT);
            memcpy(fg_gsmTxBuf, AT_QSSLCFG_CACERT, fg_gsmTxCount);
            break;
        //-----------------------------------------------
        case 12: // client config
            fg_gsmTxCount = sizeof(AT_QSSLCFG_CLIENT);
            memcpy(fg_gsmTxBuf, AT_QSSLCFG_CLIENT, fg_gsmTxCount);
            break;
        //-----------------------------------------------
        case 13: //user config 
            fg_gsmTxCount = sizeof(AT_QSSLCFG_USER);
            memcpy(fg_gsmTxBuf, AT_QSSLCFG_USER, fg_gsmTxCount);
            break;
        //-----------------------------------------------
        case 14: //
            fg_gsmTxCount = sizeof(AT_QSSLCFG_SECLEVEL);
            memcpy(fg_gsmTxBuf, AT_QSSLCFG_SECLEVEL, fg_gsmTxCount);
            break;
        //-----------------------------------------------
        case 15: //
            fg_gsmTxCount = sizeof(AT_QSSLCFG_SSLVER);
            memcpy(fg_gsmTxBuf, AT_QSSLCFG_SSLVER, fg_gsmTxCount);
            break;
        //-----------------------------------------------
        case 16: //
            fg_gsmTxCount = sizeof(AT_QSSLCFG_CIPHER);
            memcpy(fg_gsmTxBuf, AT_QSSLCFG_CIPHER, fg_gsmTxCount);
            break;
        //-----------------------------------------------
        case 17: //
            fg_gsmTxCount = sizeof(AT_QSSLCFG_IGNORRTC);
            memcpy(fg_gsmTxBuf, AT_QSSLCFG_IGNORRTC, fg_gsmTxCount);
            break;
        //-----------------------------------------------
        case 18:    //Start MQTT SSL Connection
            fg_gsmTxCount = sizeof(AT_QMTOPEN) - 1;
            memcpy(fg_gsmTxBuf, AT_QMTOPEN, fg_gsmTxCount);
            memcpy(&fg_gsmTxBuf[fg_gsmTxCount], MQTT_AWS_HOST, sizeof(MQTT_AWS_HOST));
            fg_gsmTxCount += sizeof(MQTT_AWS_HOST) - 1;
            
            memcpy(&fg_gsmTxBuf[fg_gsmTxCount], "\",\"", 3);
            fg_gsmTxCount += 3;
            
            memcpy(&fg_gsmTxBuf[fg_gsmTxCount], MQTT_AWS_PORT, sizeof(MQTT_AWS_PORT));
            fg_gsmTxCount += sizeof(MQTT_AWS_PORT) - 1;
            fg_gsmTxBuf[fg_gsmTxCount] = '\"';
            fg_gsmTxCount += 2;
            break;
        //-----------------------------------------------
        // there will not be any case for 19. The expected connection response will be delayed but received
        //-----------------------------------------------
        case 20:
            fg_gsmTxCount = sizeof(AT_QMTCONN);
            memcpy(fg_gsmTxBuf, AT_QMTCONN, fg_gsmTxCount);
            fg_gsmDelay = 100;      //waiting time 10 seconds
            break;
        //-----------------------------------------------
        case 21:    //subscribe
            fg_gsmTxCount = sizeof(AT_QMTSUB_PING);
            memcpy(fg_gsmTxBuf, AT_QMTSUB_PING, fg_gsmTxCount);
            fg_gsmDelay = 50;      //waiting time 10 seconds
            break;
        //-----------------------------------------------
        case 22:    //init finished case (wait) ---------
            fg_gsmTxCount = 0;      
            alua_gsmSetTimeout(0x01, 0);
            fg_gsmState = STATE_GSM_IDLE;
            if(fn_cbMqtt) fn_cbMqtt();          //if not null, callback
////            alua_gsmAttachFunction(GSM_FUNC_NONE); 
            fg_mqttInitFlag = 0x01;
            break;
        //-----------------------------------------------
        case 23:    //publish cell info
            fg_gsmTxCount = sizeof(AT_QMTPUB_CELL);
            memcpy(fg_gsmTxBuf, AT_QMTPUB_CELL, fg_gsmTxCount);
            fg_gsmDelay = 30;      //waiting time 3 seconds
            break;
        //-----------------------------------------------
        case 24:    //transmit data++
            //fg_gsmTxCount = 0;
            //memcpy(fg_gsmTxBuf, "This is a test for publish message. Hope it is received at server.", 66);
            fg_gsmTxCount = alua_gsmCellData(fg_gsmTxBuf);
            fg_gsmTxBuf[fg_gsmTxCount++] = CHAR_CTRL_Z;
            alua_gsmSetTimeout(0x01, 0);
            alua_uart2Transmit(fg_gsmTxBuf, fg_gsmTxCount);  
            
            fg_gsmTxCount = 0;
            break;
    }
    
    memset(fg_gsmRxBuf, 0, sizeof(fg_gsmRxBuf));
    
    if(fg_gsmTxCount)
    {           
        alua_gsmSetTimeout(0x01, fg_gsmDelay);       //setting 1 second timeout
          
        fg_gsmState = STATE_GSM_IDLE;
        fg_gsmTxBuf[fg_gsmTxCount - 1 ] = 0x0D;
        fg_gsmTxBuf[fg_gsmTxCount] = 0x0A;
        fg_gsmTxBuf[fg_gsmTxCount + 1] = 0x00;
        fg_gsmTxCount += 1;
        alua_uart2Transmit(fg_gsmTxBuf, fg_gsmTxCount);           
    }
}

//============================================================================
//returns MQTT Initilize Flag 
uint8_t alua_mqqtIsInitialized(void)
{
    return(fg_mqttInitFlag);
}

//============================================================================
//to publish any topic
void alua_gsmMqttPublish(void(*a_pfnCB)(void))
{
    fn_cbMqtt = a_pfnCB;
    fg_gsmCurrStage = 23;
    fg_gsmState = STATE_GSM_INIT;
}

//============================================================================
//this function will fill certificate to buffer for as per a_certType and 
//returns number of bytes (16 bits)
uint16_t alua_gsmMqttCopyCert(en_certType a_certType)
{
    uint8_t *l_pHeader = 0, *l_pFooter = 0, *l_pCert = 0;
    uint8_t l_chunkCount = 0, l_headerCount, l_footerCount = 0;
    uint16_t l_index = 0, l_certLen = 0;

    memset(fg_gsmTxBuf, 0, UART_GSM_BUFFER_LEN);
    
    if(a_certType == AWS_CERT_CA1)
    {
        l_pHeader = MQTT_AWS_CA1_HEADER;
        l_pFooter = MQTT_AWS_CA1_FOOTER;
        l_pCert = MQTT_AWS_CA1_CERT;
        l_headerCount = sizeof(MQTT_AWS_CA1_HEADER) - 1;
        l_footerCount = sizeof(MQTT_AWS_CA1_FOOTER) - 1;
        l_certLen = sizeof(MQTT_AWS_CA1_CERT) - 1;
    }
    else if(a_certType == AWS_CERT_CLIENT)
    {
        l_pHeader = MQTT_AWS_CLIENT_HEADER;
        l_pFooter = MQTT_AWS_CLIENT_FOOTER;
        l_pCert = MQTT_AWS_CLIENT_CERT;        
        l_headerCount = sizeof(MQTT_AWS_CLIENT_HEADER) - 1;
        l_footerCount = sizeof(MQTT_AWS_CLIENT_FOOTER) - 1;
        l_certLen = sizeof(MQTT_AWS_CLIENT_CERT) - 1;
    }
    else if(a_certType == AWS_CERT_PRIVATE)
    {
        l_pHeader = MQTT_AWS_PRIVATE_HEADER;
        l_pFooter = MQTT_AWS_PRIVATE_FOOTER;
        l_pCert = MQTT_AWS_PRIVATE_CERT;        
        l_headerCount = sizeof(MQTT_AWS_PRIVATE_HEADER) - 1;
        l_footerCount = sizeof(MQTT_AWS_PRIVATE_FOOTER) - 1;
        l_certLen = sizeof(MQTT_AWS_PRIVATE_CERT) - 1;
    }
    else
    {
        return(0);
    }
  
    //header
    memcpy(&fg_gsmTxBuf[l_index], l_pHeader, l_headerCount);
    l_index += l_headerCount;
    fg_gsmTxBuf[l_index++] = 0x0A;
    
    //copying certificate
    do 
    {
        l_chunkCount = (l_certLen >= CERT_CHUNK_LEN ? CERT_CHUNK_LEN : l_certLen);
        memcpy(&fg_gsmTxBuf[l_index], l_pCert, l_chunkCount);

        l_pCert += l_chunkCount;
        l_certLen  -= l_chunkCount;
        l_index += l_chunkCount;
        fg_gsmTxBuf[l_index++] = 0x0A;
        
    }while (l_certLen);
    
    //footer
    memcpy(&fg_gsmTxBuf[l_index], l_pFooter, l_footerCount);
    l_index += l_footerCount;
    fg_gsmTxBuf[l_index++] = 0x0A;
    
    return(l_index);
}

